@@ -22,6 +22,7 @@ import testflows.settings as settings
 from .compress import CompressedFile
 from .transform.log.pipeline import RawLogPipeline
 from .transform.log.pipeline import NiceLogPipeline
+from .transform.log.pipeline import BriskLogPipeline
 from .transform.log.pipeline import DotsLogPipeline
 from .transform.log.pipeline import ShortLogPipeline
 from .transform.log.pipeline import SlickLogPipeline
@@ -121,6 +122,14 @@ def stdout_nice_handler():
         log.seek(0)
         NiceLogPipeline(log, sys.stdout, tail=True, show_input=False).run()
 
+def stdout_brisk_handler():
+    """Handler to output messages to sys.stdout
+    using "brisk" format.
+    """
+    with CompressedFile(settings.read_logfile, tail=True) as log:
+        log.seek(0)
+        BriskLogPipeline(log, sys.stdout, tail=True, show_input=False).run()
+
 def stdout_manual_handler():
     """Handler to output messages to sys.stdout
     using "manual" format.
@@ -151,6 +160,7 @@ def start_output_handler():
         "fails": stdout_fails_handler,
         "new-fails": stdout_new_fails_handler,
         "nice": stdout_nice_handler,
+        "brisk": stdout_brisk_handler,
         "quiet": stdout_silent_handler,
         "short": stdout_short_handler,
         "dots": stdout_dots_handler,
