@@ -127,7 +127,7 @@ def format_attribute(msg, no_colors=False):
         out = [f"{_indent}{' ' * 2}{color_secondary_keyword('Attributes', no_colors=no_colors)}"]
 
     out.append(color(f"{_indent}{' ' * 4}{msg['attribute_name']}", "white", attrs=["dim"], no_colors=no_colors))
-    out.append(color(f"{textwrap.indent(msg['attribute_value'], prefix=(_indent + ' ' * 6))}", "white", attrs=["dim"], no_colors=no_colors))
+    out.append(color(f"{textwrap.indent(str(msg['attribute_value']), prefix=(_indent + ' ' * 6))}", "white", attrs=["dim"], no_colors=no_colors))
     return "\n".join(out) + "\n"
 
 def format_tag(msg, no_colors=False):
@@ -300,4 +300,4 @@ def transform(no_colors=False, show_input=True):
                         last_message[0] = msg
             else:
                 line = None
-        line = yield line
\ No newline at end of file
+        line = yield line
